#include "DLList.h"

/* Creates an empty list
	Parameters: -
	Returns:
		pointer to header of newly created list
*/
ListT createEmpty()
{
  ListHeaderT *p;
  
  if ( NULL == (p = (ListHeaderT *)(malloc(sizeof(NodeT)))))
  {
		fprintf(stderr, "createEmpty: out of space\n\n");
		return NULL;
	}
	p->first = NULL;
	p->last = NULL;
	return p;
}
/* Creates new node of key "givenKey"
	Parameters:
		givenKey - key to find
	Returns:
		pointer to node with given key if successful
		or NULL if not
*/
NodeT *createNode( const KeyT givenKey )
{
  NodeT *p;
  
  if ( NULL == (p = (NodeT *)(malloc(sizeof(NodeT)))))
  {
		fprintf(stderr, "createNode: out of space\n\n");
		return NULL;
	}
  p->key = givenKey;
  return p;
}
NodeT *findFromFront( const KeyT givenKey, const ListT L)
/* Finds node with key "givenKey" starting at front of list
	Parameters:
		givenKey - key to find
		L - list to search for given key
	Returns:
		pointer to node with given key if successful
		or NULL if not
*/
{
	NodeT *p;

	if ( L == NULL )
	{
		fprintf(stderr, "findFromFront: NULL pointer given for List\n\n");
		return NULL;
	}
	p = L->first;
	while ( p != NULL ) /* not at end of list */
  	if ( p->key == givenKey )
    {
    	return p; /* key found at address p */
    }
    else
    	p = p->next;
  return NULL; /* not found */
}
void insertAtFront( NodeT *p, ListT L)
/* Inserts node pointed to by p at the beginning of list L
	Parameters:
		p - pointer to node to insert
		L - list onto which to insert
	Returns: -
*/
{
	if (L == NULL) 
	{
		fprintf(stderr, "insertAtFront: NULL pointer given for List\n");
		return;
	}
	if ( L->first == NULL )
	{
		L->first = p;
		L->last = p;
		p->next = NULL;
		p->prev = NULL;
	}
	else
	{
		p->next = L->first;
		L->first->prev = p;
		p->prev = NULL;
		L->first = p;
	}
}
void insertAtEnd( NodeT *p, ListT L)
/* Inserts node pointed to by p at the end of list L
	Parameters:
		p - pointer to node to insert
		L - list onto which to insert
	Returns: -
*/
{
	if (L == NULL) 
	{
		fprintf(stderr, "insertAtEnd: NULL pointer given for List\n");
		return;
	}
	if ( L->first == NULL )
	{
		L->first = p;
		L->last = p;
		p->next = NULL;
		p->prev = NULL;
	}
	else
	{
		p->next = NULL;
		p->prev = L->last;
		L->last->next = p;
		L->last = p;
	}
}
/* Inserts node pointed to by p before the node of key "givenKey" in list L
	Parameters:
		p - pointer to node to insert
		L - list onto which to insert
		givenKey - key of node before which to insert
	Returns: -
*/
void insertBeforeKey( KeyT givenKey, NodeT *p, ListT L )
{
	NodeT* q;

	q = findFromFront( givenKey, L);
	if ( q != NULL )
	{
	 /* node with key givenKey has address q */
	  if ( q == L->first )
	  { /* insert before first node */
	    p->next = q;
			p->prev = NULL;
			q->prev = p;
	    L->first = p;
	  }
	  else
	  {
			p->prev = q->prev;
	    p->next = q;
	    q->prev->next = p;
			q->prev = p;
	  }
	}
}
/* Inserts node pointed to by p after the node of key "givenKey" in list L
	Parameters:
		p - pointer to node to insert
		L - list onto which to insert
		givenKey - key of node after which to insert
	Returns: -
*/
void insertAfterKey( KeyT givenKey, NodeT *p, ListT L )
{
	NodeT* q;

	q = findFromFront( givenKey, L);
	if ( q != NULL )
	{
	 /* node with key givenKey has address q */
	  if ( q == L->last )
	  { /* insert after last node */
			p->next = NULL;
	    p->prev = q;
			q->prev = p;
	    L->last = p;
	  }
	  else
	  {
			p->next = q->next;
			p->prev = q;
	    q->next->prev = p;
	    q->next = p;
	  }
	}
}
void deleteAtFront( const ListT L)
/* deletes the front node from list L
	Parameters:
		L - list from which to delete
	Returns: -
*/
{
  NodeT *p;
  
	if (L == NULL) 
	{
		fprintf(stderr, "deleteAtFront: NULL pointer given for List\n");
		return;
	}
	if ( L->first == NULL )
	{
	  return;
	}
	else
	{
	  p = L->first;
	  L->first = L->first->next;
		if (L->first != NULL)
			L->first->prev = NULL;
	  if (L->first == NULL )
	     L->last = NULL;
	  free( p );
	}
}
void deleteAtEnd( const ListT L)
/* Deletes last node from list L
	Parameters:
		L - list from which to delete
	Returns: -
*/
{
  NodeT *p;
  
	if (L == NULL) 
	{
		fprintf(stderr, "deleteAtBack: NULL pointer given for List\n");
		return;
	}
	if ( L->last == NULL || L->first == NULL )
	{
	  return;
	}
	else
	{
    L->last = L->last->prev;
	  free( L->last->next );
		L->last->next = NULL;
	  if (L->last == NULL ) L->first = NULL;
	}
}
/* Removes node of key "givenKey" from list L
		L - list onto which to insert
		givenKey - key of node after which to insert
	Returns:  0 - if failure, non 0 if success
*/
int deleteByKey( KeyT givenKey, ListT L )
{
	NodeT* q;

	q = findFromFront( givenKey, L);
	if ( q != NULL )
	{
		if ( q == L->first )
		{
			if ( q != L->last )
			{
				q ->next->prev = NULL;
				L->first = q->next;
			}
			else
			{ /* one element list */
				L->first = L->last = NULL;
			}
		}
		else
		if ( q == L->last)
		{
			q->prev->next = NULL;
			L->last = q->prev;
		}
		else
		{ /* more than two nodes onto list L*/
			q->prev->next = q->next;
			q->next->prev = q->prev;
		}
		free( q );
		return 1;
	}
	return 0;
}
/* Removes all nodes from list L
		L - list to delete
*/
void deleteList( ListT L )
{
  NodeT *p;
	
	if (L == NULL) 
	{
		fprintf(stderr, "deleteList: NULL pointer given for List\n");
		return;
	}
  while ( L->first != NULL )
  {
    p = L->first;
    L->first = L->first->next;
    free( p );
  }
  L->last = NULL;
}
NodeT *findFromEnd( const KeyT givenKey, const ListT L)
/* Finds node with key "givenKey" starting at rear of list
	Parameters:
		givenKey - key to find
		L - list to search for given key
	Returns:
		pointer to node with given key if successful
		or NULL if not
*/
{
	NodeT *p;

	if ( L == NULL )
	{
		fprintf(stderr, "findFromBack: NULL pointer given for List\n\n");
		return NULL;
	}
	p = L->last;
	while ( p != NULL ) /* not at end of list */
  	if ( p->key == givenKey )
    {
    	return p; /* key found at address p */
    }
    else
    	p = p->prev;
  return NULL; /* not found */
}

/* Returns pointer to first element of list L
*/
NodeT *front( ListT L )
{
  return L->first;
}
/* Returns pointer to last element of list L
*/
NodeT *back( ListT L )
{
  return L->last;
}
/* Prints all nodes of list L
*/
void printList( ListT L )
{
  NodeT *p;
  
	fprintf(fpOut, "Fwd: ");
	if ( L == NULL )
		fprintf(fpOut, "List was not created\n");
	else
	{
	  p = L->first;
		if ( p == NULL )
			fprintf(fpOut, "Empty list\n");
		else
		{
		  while ( p != NULL )
		  {
		    printf("%d ", p->key);
		    p = p->next;
		  }
		  printf("\n");
		}
	}
}
/* Prints all nodes of list L
*/
void revPrintList( ListT L )
{
  NodeT *p;

  fprintf(fpOut, "Rev: ");
	if ( L == NULL )
		fprintf(fpOut, "List was not created\n");
	else
	{
	  p = L->last;
		if ( p == NULL )
			fprintf(fpOut, "Empty list\n");
		else
		{
		  while ( p != NULL )
		  {
		    printf("%d ", p->key);
		    p = p->prev;
		  }
		  printf("\n");
		}
	}
}



