#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include <string.h>
#include <ctype.h>
#include "DLList.h"

FILE *fpIn;
FILE *fpOut;
int main(int argc, char *argv[])
{
  char buf[ 256 ];
  int lCount, key, n;
	NodeT *p;
	ListT L = NULL;
  /* ... */
  
	fpIn = stdin;
	fpOut = stdout; 	
	if ( argc > 1 )
	{ /* we have args */
		while ( --argc )
		{ /* scan argument list backwards */
			if ( *argv[ argc ] == '-' )
			{ /* hyphen indicates a parameter */
				switch ( tolower(*( argv[ argc ] + 1 )) )
				{
					case 'i':	/* next arg is input file name. Open it */
						if ( NULL != ( fpIn = fopen( argv[ argc + 1 ], "r" )))
						  break;
						if ( NULL != ( fpIn = fopen( argv[ argc ] + 2, "r" )))
						  break;
						fpIn = stdin; /* set default */
						break;
					case 'o': /* next arg is output file name. Open it */
						if ( NULL != ( fpOut = fopen( argv[ argc + 1 ], "w" )))
						  break;
						if ( NULL != ( fpOut = fopen( argv[ argc ] + 2, "w" )))
						  break;
						fpOut = stdout;  /* set default */
						break;
					default:
						/* change this accordingly */
						fprintf( stderr, "%s performs tests on snlgly linked list functions\n",
											argv[ 0 ] );
						fprintf( stderr, "Usage: %s [-i <inputFilename>] [-o <outputFilename>]\n",
											argv[ 0 ] );
						fprintf( stderr, "\t <inputFilename> defaults to stdin\n" );
						fprintf( stderr, "\t <outputFilename> defaults to stdout\n" );
            fprintf( stderr, "Input structure:\n<command><args>\n" );
            fprintf( stderr, "Arguments are separated by spaces. First arg immediately follows <command>\n" );
            fprintf( stderr, "Command is one of:\n\n" );
            fprintf( stderr, "\tcl\t- create empty list\n" );
            fprintf( stderr, "\tie\t- insert at end (append)\n\tif\t- insert at front\n" );
            fprintf( stderr, "\tibk\t- insert before key\n\tiak\t- insert after key\n" );
            fprintf( stderr, "\tf\t- find\n\tdk\t- delete by key\n" );
            fprintf( stderr, "\tdl\t- delete whole list\n\tdf\t- delete at front\n" );
            fprintf( stderr, "\tdbk\t- delete at back\n\tp\t- print list\n" );
						exit( 1 );
				}
			} /* if current arg begins with '-' */
		} /* while have args */
	} /* if prog has args */
	/* do your job */
	/* Read number of vertices in graph G */
	lCount = 0;
	while (fgets( buf, sizeof( buf ), fpIn ))
	{
		fputs( buf, fpOut );
		lCount++;
		switch( tolower(buf[0] ))
		{
			case 'i':
					switch( tolower(buf[1] ))
					{
						case 'a':
							if ( tolower(buf[2]) == 'k' )
							{
								key = atoi( &buf[3] );
								n = 3;
								while ( isspace( buf[ n ])) n++;
								while ( isdigit( buf[ n ])) n++;
								p = createNode(atoi(&buf[n]));
								if (L == NULL)
								{
									fprintf(stderr, "[%d] Cannot insert into nonexisting list\n", lCount);
									free( p );
								}
								else
									insertBeforeKey( key, p, L );
							}
							else
									fprintf(stderr, "[%d] Unknown command: %s", lCount, buf);
							break;
						case 'b':
							if ( tolower(buf[2]) == 'k' )
							{
								key = atoi( &buf[3] );
								n = 3;
								while ( isspace( buf[ n ])) n++;
								while ( isdigit( buf[ n ])) n++;
								p = createNode(atoi(&buf[n]));
								if (L == NULL)
								{
									fprintf(stderr, "[%d] Cannot insert into nonexisting list\n", lCount);
									free( p );
								}
								else
									insertBeforeKey( key, p, L );
							}
							else
									fprintf(stderr, "[%d] Unknown command: %s", lCount, buf);
							break;
						case 'e':
								p = createNode(atoi(&buf[2]));
								if (L == NULL)
								{
									fprintf(stderr, "[%d] Cannot insert into nonexisting list\n", lCount);
									free( p );
								}
								else
									insertAtEnd( p, L );
							break;
						case 'f':
								p = createNode(atoi(&buf[2]));
								if (L == NULL)
								{
									fprintf(stderr, "[%d] Cannot insert into nonexisting list\n", lCount);
									free( p );
								}
								else
									insertAtFront( p, L );
							break;
						default:
									fprintf(stderr, "[%d] Unknown command: %s", lCount, buf);
							break;
					}
				break;
			case 'c':
					switch( tolower(buf[1] ))
					{
						case 'l':
							if ( L == NULL )
								L = createEmpty();
							else
								fprintf(stderr, "[%d] List already created\n", lCount);
							break;
						default:
							break;
					}
				break;
				case 'd':
					switch ( tolower( buf[1] ))
					{
						case 'e':
							if (L == NULL)
							{
								fprintf(stderr, "[%d] Cannot delete from nonexisting list\n", lCount);
							}
							else
								deleteAtEnd( L );
							break;
						case 'f':
							if (L == NULL)
							{
								fprintf(stderr, "[%d] Cannot delete from nonexisting list\n", lCount);
							}
							else
								deleteAtFront( L );
							break;
						case 'k':
							key = atoi( &buf[2] );
							if (L == NULL)
							{
								fprintf(stderr, "[%d] Cannot delete from nonexisting list\n", lCount);
							}
							else
								deleteByKey( key, L );
							break;
						case 'l':
							if (L == NULL)
							{
								fprintf(stderr, "[%d] Cannot delete from nonexisting list\n", lCount);
							}
								deleteList( L );
							break;
						default:
									fprintf(stderr, "[%d] Unknown command: %s", lCount, buf);
					}
					break;
			case 'f':
					if (L == NULL)
					{
						fprintf(stderr, "[%d] Cannot print nonexisting list\n", lCount);
					}
					else
					{
						key = atoi( &buf[1] );
						fprintf(fpOut, "from end:%p\n", findFromEnd( key, L ));
						fprintf(fpOut, "from front:%p\n", findFromFront( key, L ));
					}
					break;
			case 'p':
					if (L == NULL)
					{
						fprintf(stderr, "[%d] Cannot print nonexisting list\n", lCount);
					}
					else
					{
						printList( L );
						revPrintList( L );
					}
				break;
			default:
				break;
		}
	}
  fclose( fpOut );
  return 0;
}
